Method: IPAccess::List#+

Defined in:
lib/ipaccess/ip_access_list.rb

#+(*addresses) ⇒ Object

Returns new instance containing elements from this object and objects passed as an argument. If objects contain IP information but it’s impossible to obtain whether they relate to black or white list, then blacklisting is assumed.

See IPAccess.to_cidrs description for more info about arguments you may pass to it.



903
904
905
906
907
# File 'lib/ipaccess/ip_access_list.rb', line 903

def +(*addresses)
  obj = self.class.new(self)
  obj.add!(*addresses)
  return obj
end