Method: IPAccess::Net::HTTP#initialize
- Defined in:
- lib/ipaccess/ghost_doc/ghost_doc_net_http.rb
#new(address) ⇒ HTTP #new(address, acl) ⇒ HTTP #new(address, port, acl) ⇒ HTTP #new(address, acl, *flags) ⇒ HTTP #new(address, port, acl, *flags) ⇒ HTTP
Creates a new object for the specified address. This method does not open the TCP connection. It optionally sets an access set given as the last parameter. If parameter is not given it sets ACL to IPAccess::Set.Global.
Flags are symbols that control behavior of IPAccess:
* +:opened_on_deny+ causes blocking method to leave a socket open when access is denied and a socket was re-checked
* +:check_only_proxy+ causes access checks to be applied only to a proxy server address if a proxy is in use
* +:check_only_real+ causes access check to be applied only to a destination address (and not to proxy server) if a proxy is in use
250 251 252 |
# File 'lib/ipaccess/ghost_doc/ghost_doc_net_http.rb', line 250 def initialize(address) # Real code hidden. end |